//=====================================================================================================
//   OpenSource code distributed under the CDDL v1.0 (http://www.opensource.org/licenses/cddl1.php)
//=====================================================================================================
//
//  Enjoy this OpenSource NinjaScript v6.0 indicator!  May it help bring you great amounts of profit!

//  If you have profited from this code, then please consider a suitable donation of any amount to its 
//  Initial Developer and/or other Contributors.  Contact information given below.

//  Also, feel free to email enhancement suggestions to:
//          Initial Developer:  SBG Trading Corp., Ben Letto, sbgtrading@yahoo.com April 11, 2008
//
//=====================================================================================================


#region Using declarations
using System;
using System.IO;
using System.Collections;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

namespace NinjaTrader.Indicator
{
	/// <summary>
	/// This indicator is a real-time indicator and does not plot against historical data. Plots the delta between buy and sell transaction volume.  Buy volume being volume trade at or above Ask, Sell volume at or below Bid.
	/// </summary>
	[Description("This indicator is a real-time indicator and does not plot against historical data. Plots a histogram splitting volume between trades at the ask or higher and trades at the bid and lower.")]
	public class CumulativeVolumeOrTick_Limit : Indicator
	{
		#region Variables
			private int activeBar = -1, Direction = 0;
			private double buys = 0;
			private double sells = 0;
			private int Vol_Lim = 50;  // Volume limit setting - indicator calculates cumulative delta of trades >= Vol_Lim value
			private bool hideBarDeltas=false;
			private CVOTL_OutputType ctype=CVOTL_OutputType.BidAskVolume;
			private CVOTL_CountLocType CounterLoc = CVOTL_CountLocType.BR; //"BottomRight" default value for CounterLocation
			private double Delta0, Delta1, Delta2, Delta3, Total, LastTransactedPrice;
			private string path = Cbi.Core.UserDataDir.ToString()+"DeltaVolume.txt";

		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
			Add(new Plot(new Pen(Color.Red, 1), PlotStyle.Dot, "DeltaDown"));
			Add(new Plot(new Pen(Color.Green, 1), PlotStyle.Dot, "DeltaUp"));
			Add(new Plot(new Pen(Color.Orange, 2), PlotStyle.Line, "Cumulative"));
			Add(new Line(Color.FromKnownColor(KnownColor.DarkSalmon), 0, "Zero"));
			DisplayInDataBox = false;
			PaintPriceMarkers = false;
			PlotsConfigurable = true;
			Plots[0].Max = 0;
			Plots[1].Min = 1;
			CalculateOnBarClose = false;
		}

		/// <summary>
		/// Called on each bar update event (incoming tick)
		/// </summary>
		protected override void OnBarUpdate()
		{
			if (CurrentBar < activeBar) return;
			else if (CurrentBar != activeBar)
			{	buys  = 0;
				sells = 0;
				Delta3 = Delta2;
				Delta2 = Delta1;
				Delta1 = Delta0;
				Delta0 = 0;
				if(CurrentBar>0) Cumulative.Set(1,Total);
				activeBar = CurrentBar;
			}

			if (Historical) Total = 0.0;
			else
			{
				string D = String.Concat(Delta3.ToString()," ",Delta2.ToString()," ",Delta1.ToString()," ",Delta0.ToString());
				if(CounterLoc==CVOTL_CountLocType.TR) DrawTextFixed("voldelta",D,TextPosition.TopRight);
				else if(CounterLoc==CVOTL_CountLocType.TL) DrawTextFixed("voldelta",D,TextPosition.TopLeft);
				else if(CounterLoc==CVOTL_CountLocType.BR) DrawTextFixed("voldelta",D,TextPosition.BottomRight);
				else if(CounterLoc==CVOTL_CountLocType.BL) DrawTextFixed("voldelta",D,TextPosition.BottomLeft);
			}
		}

        /// <summary>
        /// Called on each incoming real time market data event
        /// </summary>
        protected override void OnMarketData(MarketDataEventArgs e)
        {
            if (e.MarketDataType != MarketDataType.Last || e.MarketData.Ask == null || e.MarketData.Bid == null)
                return;
			if(ctype == CVOTL_OutputType.BidAskVolume)
			{
	            if ((e.Price >= e.MarketData.Ask.Price) && (e.Volume >= Vol_Lim))    { buys += e.Volume;  Total += e.Volume; }
    	        else if ((e.Price <= e.MarketData.Bid.Price) && (e.Volume >= Vol_Lim)) { sells += e.Volume; Total -= e.Volume; }
			}
			else if(ctype == CVOTL_OutputType.UpDownTickVolume)
			{
				if(e.Price > LastTransactedPrice) Direction = 1;
				if(e.Price < LastTransactedPrice) Direction = -1;
				if((Direction == 1) && (e.Volume >= Vol_Lim))  { buys += e.Volume;  Total += e.Volume; }
				if((Direction == -1) && (e.Volume >= Vol_Lim))  { sells += e.Volume; Total -= e.Volume; }
				LastTransactedPrice = e.Price;
			}
			else if(ctype == CVOTL_OutputType.TickCount)
			{
				if((e.Price > LastTransactedPrice) && (e.Volume >= Vol_Lim)) { buys++;  Total++; }
				if((e.Price < LastTransactedPrice) && (e.Volume >= Vol_Lim))  { sells++; Total--; }
				LastTransactedPrice = e.Price;
			}
			Delta0 = buys-sells;
			if(!hideBarDeltas)
			{
				DeltaUp.Set(Delta0); 
				DeltaDown.Set(Delta0);
			}
		}		
//===============================================================================
		#region Properties
		/// <summary>
		/// </summary>
        [Description("Hide bar DeltaUp and DeltaDown plots?")]
        [Category("Display")]
        public bool HideBarDeltas
        {
            get { return hideBarDeltas; }
            set { hideBarDeltas = value; }
        }
		/// <summary>
		/// </summary>
        [Description("Counter location (NONE,TL,TR,BL,BR)")]
        [Category("Display")]
        public CVOTL_CountLocType CounterLocation
        {
            get { return CounterLoc; }
            set { CounterLoc = value; }
        }
        [Description("Cumulative Type")]
        [Category("Parameters")]
        public CVOTL_OutputType Ctype
        {
            get { return ctype; }
            set { ctype = value; }
        }

		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries DeltaDown
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries DeltaUp
        {
            get { return Values[1]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries Cumulative
        {
            get { return Values[2]; }
        }

		#endregion
	}
}
public enum CVOTL_OutputType
{
	BidAskVolume,
	UpDownTickVolume,
	TickCount
}
public enum CVOTL_CountLocType
{	BR,
	TR,
	BL,
	TL,
	None
}
#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private CumulativeVolumeOrTick_Limit[] cacheCumulativeVolumeOrTick_Limit = null;

        private static CumulativeVolumeOrTick_Limit checkCumulativeVolumeOrTick_Limit = new CumulativeVolumeOrTick_Limit();

        /// <summary>
        /// This indicator is a real-time indicator and does not plot against historical data. Plots a histogram splitting volume between trades at the ask or higher and trades at the bid and lower.
        /// </summary>
        /// <returns></returns>
        public CumulativeVolumeOrTick_Limit CumulativeVolumeOrTick_Limit(CVOTL_OutputType ctype)
        {
            return CumulativeVolumeOrTick_Limit(Input, ctype);
        }

        /// <summary>
        /// This indicator is a real-time indicator and does not plot against historical data. Plots a histogram splitting volume between trades at the ask or higher and trades at the bid and lower.
        /// </summary>
        /// <returns></returns>
        public CumulativeVolumeOrTick_Limit CumulativeVolumeOrTick_Limit(Data.IDataSeries input, CVOTL_OutputType ctype)
        {
            checkCumulativeVolumeOrTick_Limit.Ctype = ctype;
            ctype = checkCumulativeVolumeOrTick_Limit.Ctype;

            if (cacheCumulativeVolumeOrTick_Limit != null)
                for (int idx = 0; idx < cacheCumulativeVolumeOrTick_Limit.Length; idx++)
                    if (cacheCumulativeVolumeOrTick_Limit[idx].Ctype == ctype && cacheCumulativeVolumeOrTick_Limit[idx].EqualsInput(input))
                        return cacheCumulativeVolumeOrTick_Limit[idx];

            CumulativeVolumeOrTick_Limit indicator = new CumulativeVolumeOrTick_Limit();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Ctype = ctype;
            indicator.SetUp();

            CumulativeVolumeOrTick_Limit[] tmp = new CumulativeVolumeOrTick_Limit[cacheCumulativeVolumeOrTick_Limit == null ? 1 : cacheCumulativeVolumeOrTick_Limit.Length + 1];
            if (cacheCumulativeVolumeOrTick_Limit != null)
                cacheCumulativeVolumeOrTick_Limit.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheCumulativeVolumeOrTick_Limit = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// This indicator is a real-time indicator and does not plot against historical data. Plots a histogram splitting volume between trades at the ask or higher and trades at the bid and lower.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.CumulativeVolumeOrTick_Limit CumulativeVolumeOrTick_Limit(CVOTL_OutputType ctype)
        {
            return _indicator.CumulativeVolumeOrTick_Limit(Input, ctype);
        }

        /// <summary>
        /// This indicator is a real-time indicator and does not plot against historical data. Plots a histogram splitting volume between trades at the ask or higher and trades at the bid and lower.
        /// </summary>
        /// <returns></returns>
        public Indicator.CumulativeVolumeOrTick_Limit CumulativeVolumeOrTick_Limit(Data.IDataSeries input, CVOTL_OutputType ctype)
        {
            return _indicator.CumulativeVolumeOrTick_Limit(input, ctype);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// This indicator is a real-time indicator and does not plot against historical data. Plots a histogram splitting volume between trades at the ask or higher and trades at the bid and lower.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.CumulativeVolumeOrTick_Limit CumulativeVolumeOrTick_Limit(CVOTL_OutputType ctype)
        {
            return _indicator.CumulativeVolumeOrTick_Limit(Input, ctype);
        }

        /// <summary>
        /// This indicator is a real-time indicator and does not plot against historical data. Plots a histogram splitting volume between trades at the ask or higher and trades at the bid and lower.
        /// </summary>
        /// <returns></returns>
        public Indicator.CumulativeVolumeOrTick_Limit CumulativeVolumeOrTick_Limit(Data.IDataSeries input, CVOTL_OutputType ctype)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.CumulativeVolumeOrTick_Limit(input, ctype);
        }

    }
}
#endregion
